@@ -1,5 +1,5 @@
 /* Support for printing Ada types for GDB, the GNU debugger.
-   Copyright (C) 1986-2025 Free Software Foundation, Inc.
+   Copyright (C) 1986-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -23,7 +23,6 @@
 #include "cli/cli-style.h"
 #include "typeprint.h"
 #include "ada-lang.h"
-#include <ctype.h>
 
 static int print_selected_record_field_types (struct type *, struct type *,
 					      int, int,
@@ -70,7 +69,7 @@ decoded_type_name (struct type *type)
       if (s == name_buffer)
 	return name_buffer;
 
-      if (!islower (s[1]))
+      if (!c_islower (s[1]))
 	return NULL;
 
       for (s = q = name_buffer; *s != '\0'; q += 1)
@@ -543,7 +542,7 @@ print_variant_clauses (struct type *var_type, struct type *discr_type,
 /* Assuming that field FIELD_NUM of TYPE represents variants whose
    discriminant is contained in OUTER_TYPE, print its components on STREAM.
    LEVEL is the recursion (indentation) level, in case any of the fields
-   themselves have nested structure, and SHOW is the number of levels of 
+   themselves have nested structure, and SHOW is the number of levels of
    internal structure to show (see ada_print_type).  For this purpose,
    fields nested in a variant part are taken to be at the same level as
    the fields immediately outside the variant part.  */
@@ -790,7 +789,7 @@ print_record_field_types (struct type *type, struct type *outer_type,
 					    0, type->num_fields () - 1,
 					    stream, show, level, flags);
 }
-   
+
 
 /* Print record type TYPE on STREAM.  LEVEL is the recursion (indentation)
    level, in case the element type itself has nested structure, and SHOW is
@@ -1041,8 +1040,18 @@ ada_print_type (struct type *type0, const char *varstring,
 	    gdb_printf (stream, ">");
 	  }
 	else if (ada_is_modular_type (type))
-	  gdb_printf (stream, "mod %s", 
-		      int_string (ada_modulus (type), 10, 0, 0, 1));
+	  {
+	    std::optional<ULONGEST> bound = ada_modular_bound (type);
+	    gdb_mpz modulus;
+	    if (bound.has_value ())
+	      {
+		modulus = *bound;
+		modulus += 1;
+	      }
+	    else
+	      modulus = 0;
+	    gdb_printf (stream, "mod %s", modulus.str ().c_str ());
+	  }
 	else
 	  {
 	    gdb_printf (stream, "range ");
