@@ -1,6 +1,6 @@
 /* Target description support for GDB.
 
-   Copyright (C) 2006-2022 Free Software Foundation, Inc.
+   Copyright (C) 2006-2026 Free Software Foundation, Inc.
 
    Contributed by CodeSourcery.
 
@@ -19,16 +19,13 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef TARGET_DESCRIPTIONS_H
-#define TARGET_DESCRIPTIONS_H 1
+#ifndef GDB_TARGET_DESCRIPTIONS_H
+#define GDB_TARGET_DESCRIPTIONS_H
 #include "gdbsupport/tdesc.h"
 #include "gdbarch.h"
 
 struct tdesc_arch_data;
 struct target_ops;
-/* An inferior's target description info is stored in this opaque
-   object.  There's one such object per inferior.  */
-struct target_desc_info;
 struct inferior;
 
 /* Fetch the current inferior's description, and switch its current
@@ -42,27 +39,10 @@ void target_find_description (void);
 
 void target_clear_description (void);
 
-/* Return the current inferior's target description.  This should only
-   be used by gdbarch initialization code; most access should be
-   through an existing gdbarch.  */
+/* Return INF's target description.  This should only be used by gdbarch
+   initialization code; most access should be through an existing gdbarch.  */
 
-const struct target_desc *target_current_description (void);
-
-/* Copy inferior target description data.  Used for example when
-   handling (v)forks, where child's description is the same as the
-   parent's, since the child really is a copy of the parent.  */
-
-void copy_inferior_target_desc_info (struct inferior *destinf,
-				     struct inferior *srcinf);
-
-/* Free a target_desc_info object.  */
-
-void target_desc_info_free (struct target_desc_info *tdesc_info);
-
-/* Returns true if INFO indicates the target description had been
-   supplied by the user.  */
-
-int target_desc_info_from_user_p (struct target_desc_info *info);
+const target_desc *target_current_description (inferior *inf);
 
 /* Record architecture-specific functions to call for pseudo-register
    support.  If tdesc_use_registers is called and gdbarch_num_pseudo_regs
@@ -239,7 +219,6 @@ int tdesc_register_in_reggroup_p (struct gdbarch *gdbarch, int regno,
 
 void set_tdesc_architecture (struct target_desc *,
 			     const struct bfd_arch_info *);
-void set_tdesc_osabi (struct target_desc *, enum gdb_osabi osabi);
 void set_tdesc_property (struct target_desc *,
 			 const char *key, const char *value);
 void tdesc_add_compatible (struct target_desc *,
@@ -257,4 +236,4 @@ void record_xml_tdesc (const char *xml_file,
 }
 #endif
 
-#endif /* TARGET_DESCRIPTIONS_H */
+#endif /* GDB_TARGET_DESCRIPTIONS_H */
