@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2014-2019 Free Software Foundation, Inc.
+   Copyright 2014-2026 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,9 +22,13 @@
 /* Always zero, used in breakpoint condition.  */
 volatile int global_zero;
 
+static pthread_barrier_t threads_started_barrier;
+
 void *
 child_function (void *arg)
 {
+  pthread_barrier_wait (&threads_started_barrier);
+
   while (1)
     {
       usleep (1); /* set breakpoint child here */
@@ -39,7 +43,12 @@ main (void)
   pthread_t child_thread;
   int res;
 
+  pthread_barrier_init (&threads_started_barrier, NULL, 2);
+
   res = pthread_create (&child_thread, NULL, child_function, NULL);
+
+  pthread_barrier_wait (&threads_started_barrier);
+
   sleep (2); /* set wait-thread breakpoint here */
   exit (EXIT_SUCCESS);
 }
