@@ -1,5 +1,5 @@
 /* Simulation code for the CR16 processor.
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2026 Free Software Foundation, Inc.
    Contributed by M Ranga Swami Reddy <MR.Swami.Reddy@nsc.com>
 
    This file is part of GDB, the GNU debugger.
@@ -32,12 +32,14 @@
 #include "sim-options.h"
 #include "sim-signal.h"
 
-#include "gdb/sim-cr16.h"
+#include "sim/sim-cr16.h"
 #include "gdb/signals.h"
 #include "opcode/cr16.h"
 
 #include "target-newlib-syscall.h"
 
+#include "cr16-sim.h"
+
 struct _state State;
 
 int cr16_debug;
@@ -65,7 +67,7 @@ struct hash_entry hash_table[MAX_HASH+1];
 INLINE static long
 hash(unsigned long long insn, int format)
 { 
-  unsigned int i = 4, tmp;
+  unsigned int i = 4;
   if (format)
     {
       while ((insn >> i) != 0) i +=4;
@@ -111,7 +113,7 @@ INLINE static void
 get_operands (operand_desc *s, uint64_t ins, int isize, int nops)
 {
   uint32_t i, opn = 0, start_bit = 0, op_type = 0; 
-  int32_t op_size = 0, mask = 0;
+  int32_t op_size = 0;
 
   if (isize == 1) /* Trunkcate the extra 16 bits of INS.  */
     ins = ins >> 16;
@@ -286,6 +288,7 @@ get_operands (operand_desc *s, uint64_t ins, int isize, int nops)
             OP[i] = (ins) & 0x3FFF;
             OP[++i] = (ins >> 14) & 0x1;     /* get 1 bit for index-reg.  */
             OP[++i] = (ins >> 16) & 0xF;     /* get 4 bit for reg.  */
+            break;
           case rindex7_abs20:
           case rindex8_abs20:
             OP[i] = (ins) & 0xFFFFF;
@@ -395,7 +398,6 @@ sim_open (SIM_OPEN_KIND kind, struct host_callback_struct *cb,
   struct simops *s;
   struct hash_entry *h;
   static int init_p = 0;
-  char **p;
   int i;
   SIM_DESC sd = sim_state_alloc (kind, cb);
   SIM_ASSERT (STATE_MAGIC (sd) == SIM_MAGIC_NUMBER);
@@ -405,7 +407,7 @@ sim_open (SIM_OPEN_KIND kind, struct host_callback_struct *cb,
   cb->syscall_map = cb_cr16_syscall_map;
 
   /* The cpu data is kept in a separately allocated chunk of memory.  */
-  if (sim_cpu_alloc_all (sd, 1) != SIM_RC_OK)
+  if (sim_cpu_alloc_all (sd, 0) != SIM_RC_OK)
     {
       free_state (sd);
       return 0;
